<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use Zeedhi\Framework\DataSource\FilterCriteria;

class AnaliseCusto {
	protected $utilities;
	protected $connection;

    public function __construct($connection, $utilities) {
        $this->connection  = $connection;
        $this->utilities = $utilities;
	}
	
	public function callProcedureConsPlan(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		try {
		    $cdfilcentpme = $row['cdfilcentpme'];
		    $cdtipocard = $row['typeMenu']['CDTIPOCARD'];
		    $filial = ($row['aggregated'] !== NULL) ? $row['aggregated']['CDFILIAL'] : $row['aggregated'];
		    $servico = $row['service']['CDSERVFILI'];
		    $initialDate = $row['initialDate'];
		    $finalDate = $row['finalDate'];
		    
			$consplan = $this->utilities->consumoPlanPMe($cdfilcentpme, $cdtipocard, $filial, $servico, $initialDate, $finalDate, 'T', 'P');
			$response->addDataSet(new DataSet("consplan", array($consplan)));
		} catch (\Exception $e) {
	    	$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function getColorSafPrat(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		$arrColors = array();
		$query = "SELECT NMSINASAFRA, NRCORSINSAFR, NRORG FROM SINASAFRA WHERE VRLIMINFSAFR < :P_PERCSAFRA AND VRLIMSUPSAFR >= :P_PERCSAFRA OR :P_PERCSAFRA = 0 AND VRLIMINFSAFR = :P_PERCSAFRA";
		
		foreach ($row as $plate) {
	    $params = array('P_PERCSAFRA' => $plate['SAFRAPORCENT']);
			$stmt = $this->connection->prepare($query);
			$stmt->execute($params);
			$resp = $stmt->fetchAll();
			
			$color = (count($resp) > 0) ? $resp[0]['NRCORSINSAFR'] : null;
			array_push($arrColors, $color);
		}
		$response->addDataSet(new DataSet("colors", $arrColors));
	}
}