<?php
namespace PME\Controllers\Servico;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class ConsPorServicoController extends \PME\Services\TeknisaCrud {
  protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
	protected $lista;
	protected $environment;
	protected $entityManager;
	protected $connection;

  public function __construct($crudService, $parameterBag, $lista, $environment, $entityManager, $connection) {
      parent::__construct($crudService, $parameterBag);
      $this->lista         = $lista;
      $this->environment   = $environment;
      $this->entityManager = $entityManager;
      $this->connection    = $connection;
	}

  public function saveConsPadr(DTO\Request $request, DTO\Response $response) {
    try {
        $this->connection->beginTransaction();
        $rows = (array) $request->getRow();

        $dataSourceName = 'consServPme';
        $this->callSave($dataSourceName, $rows);
        $this->connection->commit();
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function saveSemCons(DTO\Request $request, DTO\Response $response) {
    try {
        $this->connection->beginTransaction();
        $rows = (array) $request->getRow();
        $rowsToSave = [];

        foreach($rows as $row) {
          if ($row['PERCAPITA'] == 'S' && $row['FIXO'] == 'S') $row['IDTPCONSPME'] = 'A';
          else if ($row['PERCAPITA'] == 'S') $row['IDTPCONSPME'] = 'P';
          else $row['IDTPCONSPME'] = 'F';
          
          array_push($rowsToSave, $row);
        }
        $this->deleteConsServ($rowsToSave);

        $dataSourceName = 'diaSemConsPme';
        $this->callSave($dataSourceName, $rowsToSave);
        $this->connection->commit();
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  private function deleteConsServ($rows) {
    foreach($rows as $row){
      if ($row['IDTPCONSPME'] == 'A') $tipo = "'P','F'";
      else if ($row['IDTPCONSPME'] == 'F') $tipo = "'F'";
      else $tipo = "'P'";
      
      $query = "DELETE CONSSERVPME WHERE CDFILCENTPME = :CDFILCENTPME AND CDSERVFILI = :CDSERVFILI AND CDFILIAL =:CDFILIAL 
                      AND DTCONSERVPME = :DTCONSERVPME AND IDQTDECONPME IN ($tipo)";

      $params = array(
        'CDFILCENTPME'=> $row['CDFILCENTPME'],
        'CDSERVFILI'  => $row['CDSERVFILI'],
        'CDFILIAL'    => $row['CDFILIAL'],
        'DTCONSERVPME' => $row['DTSEMCONSPME'],
      );

      $this->connection->prepare($query)->execute($params);
    }        
  }

  private function callSave($dataSourceName, $rowsToSave){
    $dataSet = new DataSet($dataSourceName, $rowsToSave);
    $request = new DTO\Request\DataSet($dataSet, "", "", "");  
    $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
    $this->crudService->save($dataSourceName, $rowsToSave);
  }
  
  public function insertListaExport(DTO\Request $request, DTO\Response $response) {
    try {
      $rows = $request->getRow();
      $nrorg = $this->environment->getNrOrgTrab();

      $unidadeServico = [];
      $servicos = [];
      
      foreach($rows['SERVICO'] as $row){
        $unidServ = $row['UNIDSERV'];
        $concatServ = $row['CDSERVFILI'];
        array_push($unidadeServico, $unidServ);
        array_push($servicos, $concatServ);
      }
      
      $rows['NRLISTA'] = $this->lista->geraLista(join("_", $unidadeServico), $nrorg);

      if(empty($rows['SERVICO'])) $rows['CDFILISERV'] = 'T';
      else if(is_array($rows['SERVICO'])) $rows['CDFILISERV'] = join("_", $servicos);
      
      if(empty($rows['UNIDADE'])) $rows['CDFILIAL'] = 'T';
      else if (is_array($rows['UNIDADE'])) $rows['CDFILIAL'] = join("_", $rows['UNIDADE']);
      
      $this->insertListaProdutos($rows);
      
      if($rows['IDQTDECONS'] === 'Fixo') $rows['IDQTDECONS'] = 'F';
      else $rows['IDQTDECONS'] = 'P';
    
      if($rows['SOBREPOR'] === '0') $rows['SOBREPOR'] = NULL;
      else $rows['SOBREPOR'] = 'S';
      
      $arrayConsGrava = $rows['IDTIPOCONSGRAVA'];
      unset($arrayConsGrava[0]);
      
      foreach($arrayConsGrava as $pos){
        $this->procedure($rows, $pos);
      }
    } catch(\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function insertListaProdutos($rows){
    $nrorg = $this->environment->getNrOrgTrab();

    $arrayProd = [];
    foreach($rows['PRODUTOS'] as $row){
      $produtos = $row['CDARVPROD'];
      array_push($arrayProd, $produtos);
    }
    
    $concatProd = join("_", $arrayProd);
    $rows['NRLISTAPROD'] = $this->lista->geraLista($concatProd, $nrorg);
  }
  
  private function procedure($rows, $pos){
    $connection = $this->entityManager->getConnection();
    $procedureCons = new StoredProcedure($connection, 'EXPORTA_CONSUMO');
  
    $procedureCons->addParam(new Param('P_CDFILICENTGRAVA', Param::PARAM_INPUT, $rows['CENTRAL'],                     Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_IDTIPOCONSGRAVA', Param::PARAM_INPUT, $pos,                                 Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_DTCONSSERVGRAVA', Param::PARAM_INPUT, $rows['DTCONSSERVGRAVA'],             Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_CDFILICENTORIG',  Param::PARAM_INPUT, $rows['PRODUTOS'][0]['CDFILCENTPME'], Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_CDFILIORIG',      Param::PARAM_INPUT, $rows['PRODUTOS'][0]['CDFILIAL'],     Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_CDSERVORIG',      Param::PARAM_INPUT, $rows['PRODUTOS'][0]['CDSERVFILI'],   Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_IDQTDECONS',      Param::PARAM_INPUT, $rows['IDQTDECONS'],                  Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_IDTIPOCONS',      Param::PARAM_INPUT, $rows['PRODUTOS'][0]['IDTIPOCONPME'], Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_DTCONSSERV',      Param::PARAM_INPUT, $rows['PRODUTOS'][0]['DTCONSERVPME'], Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_NRLISTA',         Param::PARAM_INPUT, $rows['NRLISTAPROD'],                 Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_NRLISTAFILISERV', Param::PARAM_INPUT, $rows['NRLISTA'],                     Param::PARAM_TYPE_STR)); 
    $procedureCons->addParam(new Param('P_SOBREPOR',        Param::PARAM_INPUT, $rows['SOBREPOR'],                    Param::PARAM_TYPE_STR)); 
    $procedureCons->execute();
  }
}
